#!/usr/bin/env python3.10
import logging as log
from os import path

# custom libs
from .env_config import OS, PWD, load_yaml
from .console_logging import print_header, print_msg
from .console_logging import print_sub_header as sub_header
from .subproc import subproc


def brew_install_upgrade(package_groups=['default']):
    """
    Run the install/upgrade of packages managed by brew, also updates brew
    Always installs the .Brewfile (which has libs that work on both mac/linux)
    Accepts args:
        * os     - string arg of either Darwin or Linux
        * devops - bool, installs devops brewfile, defaults to false
    """
    brew_msg = '[green][b]brew[/b][/] app Installs/Upgrades'
    print_header('🍺 ' + brew_msg)

    install_cmd = "brew bundle --quiet"

    if 'default' in package_groups:
        subproc(['brew update --quiet',
                 'brew upgrade --quiet',
                 f'{install_cmd} --global'])

    # install os specific or package group specific brew stuff
    brewfile = path.join(PWD, 'config/brew/Brewfile_')

    # sometimes there isn't an OS specific brewfile, but there always is 4 mac
    os_brewfile = path.exists(brewfile + OS[0])

    if os_brewfile or package_groups:
        install_cmd += f" --file={brewfile}"

        if os_brewfile:
            os_msg = f'[b]{OS[0]}[/b] specific package installs...'
            sub_header(os_msg)
            subproc([f'{install_cmd}{OS[0]}'], error_ok=True)
            print_msg(f'{OS[0]} specific packages installed.')

        if package_groups:
            for group in package_groups:
                group_file = brewfile + group
                if group != "default" and path.exists(group_file):
                    # Installing devops specific brew app Installs/Upgrades
                    msg = f"{group.title()} specific package installs..."
                    sub_header(msg)
                    subproc([f'{install_cmd}{group}'], error_ok=True)
                    print_msg(f'{group.title()} specific packages installed.')

    # cleanup operation doesn't seem to happen automagically :shrug:
    sub_header('[b]brew[/b] final cleanup')
    subproc(['brew cleanup'])
    print_msg('Cleanup completed.')
    return


def run_pkg_mngrs(pkg_mngrs=[], pkg_groups=[]):
    """
    Installs packages with apt, brew, pip3.10, snap, flatpak. If no pkg_mngrs
    list passed in, only use brew/pip3.10 for mac. Takes optional variable,
    pkg_group_lists to install optional packages.
    Returns True.
    """
    pkg_mngrs_list_of_dicts = load_yaml(path.join(PWD, 'config/packages.yml'))
    log.debug(f"pkg_mngrs: {pkg_mngrs}", extra={"markup": True})
    log.debug(f"pkg_groups: {pkg_groups}", extra={"markup": True})

    # we iterate through pkg_mngrs which should already be sorted
    for pkg_mngr in pkg_mngrs:

        # brew has a special flow with "Brewfile"s
        if pkg_mngr == 'brew':
            if any(check in pkg_groups for check in ['devops', 'default']):
                brew_install_upgrade(pkg_groups)
            # skip everything below because install process already covered
            continue

        pkg_mngr_dict = pkg_mngrs_list_of_dicts[pkg_mngr]
        required_pkg_groups = pkg_mngr_dict['packages']

        debug_line = f"pkg groups for {pkg_mngr} are {required_pkg_groups}"
        log.debug(debug_line, extra={"markup": True})

        # make sure that the package manage has any groups that were passed in
        if any(check in pkg_groups for check in required_pkg_groups):

            pkg_emoji = pkg_mngr_dict['emoji']
            msg = f'{pkg_emoji} [green][b]{pkg_mngr}[/b][/] app Installs'
            print_header(msg)
            print('')

            # run package manager specific setup if needed, & updates/upgrades
            pkg_cmds = pkg_mngr_dict['commands']
            log.debug(f"package manager commands are: {pkg_cmds}")
            for pre_cmd in ['setup', 'update', 'upgrade']:
                if pre_cmd in pkg_cmds:
                    SPINNER = True
                    if 'sudo' in pkg_cmds[pre_cmd]:
                        SPINNER = False
                    subproc([pkg_cmds[pre_cmd]], spinner=SPINNER)
                    sub_header(f"[b]{pre_cmd.title()}[/b] completed.")

            # list of actually installed packages
            installed_pkgs = subproc([pkg_cmds['list']], quiet=True)

            for pkg_group in pkg_groups:
                if pkg_group in required_pkg_groups:
                    install_pkg_group(installed_pkgs,
                                      required_pkg_groups[pkg_group],
                                      pkg_cmds['install'])

                    sub_header(f'{pkg_group.title()} packages installed.')

            if 'cleanup' in pkg_cmds:
                subproc([pkg_cmds['cleanup']])
                sub_header("[b]Cleanup[/b] step Completed.")
    return


def install_pkg_group(installed_pkgs=[], pkgs_to_install=[], install_cmd=""):
    """
    installs packages if they are not already intalled with intall_cmd
    Returns True.
    """
    SPINNER = True
    install_pkg = False
    # the spinner status thing rich provides breaks with input
    if 'sudo' in install_cmd:
        SPINNER = False

    if 'upgrade' in install_cmd or not installed_pkgs:
        install_pkg = True

    log.debug(f"pkgs_to_install are {pkgs_to_install}",
              extra={"markup": True})
    for pkg in pkgs_to_install:
        if installed_pkgs:
            if pkg not in installed_pkgs:
                install_pkg = True
        if install_pkg:
            subproc([install_cmd + pkg], quiet=True, spinner=SPINNER)
    return True



#!/usr/bin/env python3.10
# Onboarding script for macOS and Debian by jessebot@Linux.com
import logging as log
from os import getenv, path, uname
# rich helps pretty print everything
from rich.prompt import Confirm
from .console_logging import print_panel
import yaml


def load_yaml(yaml_config_file=""):
    """
    load config yaml files for onboardme and return as dicts
    """
    if path.exists(yaml_config_file):
        with open(yaml_config_file, 'r') as yaml_file:
            return yaml.safe_load(yaml_file)
    else:
        # print(f"Config file we got was not present: {yaml_config_file}")
        return None


# pathing
PWD = path.dirname(__file__)
HOME_DIR = getenv("HOME")

# defaults
DEFAULTS = load_yaml(f"{PWD}/config/onboardme_config.yml")
USR_CONFIG_FILE = load_yaml(f'{HOME_DIR}/.config/onboardme/config.yaml')

# env
SYSINFO = uname()
# this will be something like ('Darwin', 'x86_64')
OS = (SYSINFO.sysname, SYSINFO.machine)


def get_version():
    """
    get the version of onboardme, print it, and return True
    """
    # this is hardcoded because we're about upgrade major versions & grab
    # pyproject.toml version info with python3.11
    print("\n🎉 v0.14.21\n")
    return True


def check_os_support():
    """
    verify we're on a supported OS and ask to quit if not.
    """
    if OS[0] not in ('Linux', 'Darwin'):
        msg = (f"[ohno]{OS}[/ohno] isn't officially supported. We"
               " have only tested Debian, Ubuntu, and macOS.")
        print_panel(msg, "⚠️  [warn]WARNING")

        quit_y = Confirm.ask("🌊 You're in uncharted waters. Wanna quit?")
        if quit_y:
            print_panel("That's probably safer. Have a safe day, friend.",
                        "Safety Award ☆ ")
            quit()
        else:
            print_panel("[red]Yeehaw, I guess.", "¯\\_(ツ)_/¯")
    else:
        print_panel("Operating System and Architechure [green]supported ♥",
                    "[cornflower_blue]Compatibility Check")


def process_steps(steps=[], firewall=False, browser=False):
    """
    process which steps to run for which OS, which steps the user passed in,
    and then make sure dependent steps are always run.

    Returns a list of str type steps to run.

    # TODO: if 'capslock_to_control' in steps: map_caps_to_control()
    """
    if steps:
        steps_list = list(steps)
        # setting up vim is useless if we don't have a .vimrc
        if 'vim_setup' in steps_list and 'dot_files' not in steps_list:
            steps_list.append('dot_files')
        if browser:
            steps_list.append('browser_setup')
        if firewall and 'Linux' in OS:
            # currently don't have a great firewall on macOS, sans lulu
            steps_list.append('firewall_setup')

    removed_duplicates = set(steps_list)
    steps = list(removed_duplicates)
    default_order = DEFAULTS['steps'][OS[0]]

    # Rearrange list by other list order Using list comprehension
    result_steps = [ele for ele in default_order if ele in steps]

    return result_steps


def sort_pkgmngrs(package_managers_list=[]):
    """
    make sure the package managers are in the right order 🤦
    """
    final_list = []
    package_managers = ['brew', 'pip3.10', 'apt', 'snap', 'flatpak']
    for mngr in package_managers:
        if mngr in package_managers_list:
            final_list.append(mngr)

    return final_list


def fill_in_defaults(defaults={}, user_config={}, always_prefer_default=False):
    """
    Compares/Combines a default dict and another dict. Prefer default values
    only if the value is empty in the second dict. Then return new dict.
    """
    for key, value in user_config.items():
        # we have to iterate through the entire config file, and who knows how
        # many levels there are, so we use recursion of this function
        if type(value) is dict:
            result_config = fill_in_defaults(defaults[key], user_config[key],
                                             always_prefer_default)
            user_config[key] = result_config

        if not value or always_prefer_default:
            user_config[key] = defaults[key]

    return user_config


def process_configs(overwrite=False, repo="", git_branch="", pkg_mngrs=[],
                    pkg_groups=[], firewall=False, remote_host="", steps=[]):
    """
    process the config in ~/.config/onboardme/config.yaml if it exists,
    then process the cli dict, and fill in defaults for anything not explicitly
    defined. Returns full final config as dict for use in script.
    """
    if remote_host:
        firewall = True
        if type(remote_host) is str:
            remote_host = [remote_host]

    cli_dict = {'package': {'managers': {OS[0]: pkg_mngrs},
                            'groups': pkg_groups},
                'log': {'file': None, 'level': ""},
                'remote_hosts': remote_host,
                'firewall': firewall,
                'steps': {OS[0]: steps},
                'dot_files': {'overwrite': overwrite,
                              'git_url': repo, 'git_branch': git_branch}}

    log.debug(f"cli_dict is:\n{cli_dict}\n", extra={"markup": True})
    if USR_CONFIG_FILE:
        log.debug(f"🗂 ⚙️  user_config_file: \n{USR_CONFIG_FILE}\n",
                  extra={"markup": True})

        usr_cfgs = fill_in_defaults(DEFAULTS, USR_CONFIG_FILE)
        log.debug("after user_config_file filled in with defaults: " +
                  f"{usr_cfgs}", extra={"markup": True})

        final_defaults = fill_in_defaults(cli_dict, usr_cfgs, True)
    else:
        final_defaults = fill_in_defaults(DEFAULTS, cli_dict)

    log.debug(" final config after filling cli_dict in with defaults:\n"
              f"{final_defaults}\n", extra={"markup": True})

    valid_steps = process_steps(final_defaults['steps'][OS[0]],
                                final_defaults['remote_hosts'])
    final_defaults['steps'][OS[0]] = valid_steps
    sorted_mngrs = sort_pkgmngrs(final_defaults['package']['managers'][OS[0]])
    final_defaults['package']['managers'][OS[0]] = sorted_mngrs

    return final_defaults



#!/usr/bin/env python3.10
"""
    NAME:           Onboardme
    DESCRIPTION:    Program to take care of a bunch of onboarding tasks for new
                    machines running macOS and/or Debian.
    AUTHOR:         Jesse Hitch
    LICENSE:        GNU AFFERO GENERAL PUBLIC LICENSE
"""
from click import option, command, Choice
import importlib
import logging

# rich helps pretty print everything
from rich.console import Console
from rich.logging import RichHandler

# custom libs
from .help_text import RichCommand, options_help
from .env_config import check_os_support, OS, process_configs, \
                        USR_CONFIG_FILE, get_version
from .env_config import DEFAULTS as OPTS
from .console_logging import print_manual_steps


HELP = options_help()


def setup_logger(level="", log_file=""):
    """
    Sets up rich logger and stores the values for it in a db for future import
    in other files. Returns logging.getLogger("rich")
    """
    if not level:
        if USR_CONFIG_FILE and 'log' in USR_CONFIG_FILE:
            level = USR_CONFIG_FILE['log']['level']
        else:
            level = 'warn'

    if not log_file:
        if USR_CONFIG_FILE:
            log_file = USR_CONFIG_FILE['log'].get('file', None)

    log_level = getattr(logging, level.upper(), None)
    # these are params to be passed into logging.basicConfig
    log_opts = {'level': log_level,
                'format': "%(message)s",
                'datefmt': "[%X]",
                'handlers': [RichHandler(rich_tracebacks=True)]}

    if log_level == 10:
        # log the name of the function if we're in debug mode :)
        log_opts['format'] = "[bold]%(funcName)s()[/bold]: %(message)s"

    # we only log to a file if one was passed into config.yaml or the cli
    if log_file:
        log_opts['handlers'] = [RichHandler(console=Console(file=log_file),
                                            rich_tracebacks=True)]

    # this uses the log_opts dictionary as parameters to logging.basicConfig()
    logging.basicConfig(**log_opts)
    return logging.getLogger("rich")


# @option('--quiet', '-q', is_flag=True, help=HELP['quiet'])
# Click is so ugly, and I'm sorry we're using it for cli parameters here, but
# this allows us to use rich.click for pretty prettying the help interface
# each of these is an option in the cli and variable we use later on
@command(cls=RichCommand)
@option('--log_level', '-l', metavar='LOGLEVEL', help=HELP['log_level'],
        type=Choice(['debug', 'info', 'warn', 'error']))
@option('--log_file', '-o', metavar='LOGFILE', help=HELP['log_file'])
@option('--steps', '-s', metavar='STEP', multiple=True,
        type=Choice(OPTS['steps'][OS[0]]), help=HELP['steps'])
@option('--git_url', '-u', metavar='URL', help=HELP['git_url'])
@option('--git_branch', '-b', metavar='BRANCH', help=HELP['git_branch'])
@option('--overwrite', '-O', is_flag=True, help=HELP['overwrite'])
@option('--pkg_managers', '-p', metavar='PKG_MANAGER', multiple=True,
        type=Choice(OPTS['package']['managers'][OS[0]]),
        help=HELP['pkg_managers'])
@option('--pkg_groups', '-g', metavar='PKG_GROUP', multiple=True,
        type=Choice(['default', 'gaming', 'media', 'devops']),
        help=HELP['pkg_groups'])
@option('--firewall', '-f', is_flag=True, help=HELP['firewall'])
@option('--remote_host', '-r', metavar="IP_ADDR", multiple=True,
        help=HELP['remote_host'])
@option('--version', is_flag=True, help=HELP['version'])
def main(log_level: str = "",
         log_file: str = "",
         steps: str = "",
         git_url: str = "",
         git_branch: str = "",
         overwrite: bool = False,
         pkg_managers: str = "",
         pkg_groups: str = "",
         firewall: bool = False,
         remote_host: str = "",
         version: bool = False):
    """
    Uses config in the script repo in config/packages.yml and config/config.yml
    If run with no options on Linux, it will install brew, pip3.10, apt,
    flatpak, and snap packages. On mac, it only installs brew/pip3.10 packages.
    config loading tries to load: cli options and then .config/onboardme/*
    """
    # return the version if that's all they want
    if version:
        return get_version()
    # before we do anything, we need to make sure this OS is supported
    check_os_support()

    # setup logging immediately
    log = setup_logger(log_level, log_file)

    # then process any local user config files, cli opts, and defaults
    usr_pref = process_configs(overwrite, git_url, git_branch, pkg_managers,
                               pkg_groups, firewall, remote_host, steps)

    log.debug(f"User passed in the following preferences:\n{usr_pref}\n",
              extra={"markup": True})

    # actual heavy lifting of onboardme happens in these
    for step in usr_pref['steps'][OS[0]]:

        if step == 'dot_files':
            from .dot_files import setup_dot_files
            # this creates a live git repo out of your home directory
            df_prefs = usr_pref['dot_files']
            setup_dot_files(OS, df_prefs['overwrite'],
                            df_prefs['git_url'], df_prefs['git_branch'])

        elif step == 'packages':
            from .pkg_management import run_pkg_mngrs
            pkg_mngrs = usr_pref['package']['managers'][OS[0]]
            pkg_groups = usr_pref['package']['groups']
            run_pkg_mngrs(pkg_mngrs, pkg_groups)

        elif step in ['vim_setup', 'neovim_setup', 'font_setup']:
            # import step from ide_setup.py in same directory
            importlib.import_module('onboardme.ide_setup', package=f'.{step}')
            func = getattr(ide_setup, step)
            func()

    if 'firewall_setup' in steps:
        from .firewall import configure_firewall
        configure_firewall(remote_host)

    print_manual_steps()
    return


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['onboardme']

package_data = \
{'': ['*'],
 'onboardme': ['config/*',
               'config/brew/*',
               'config/firewall/*',
               'config/firewall/ufw/*',
               'config/terminal.sexy/*']}

install_requires = \
['GitPython>=3.1.29,<4.0.0',
 'PyYAML>=6.0,<7.0',
 'click>=8.1.3,<9.0.0',
 'rich>=12.6.0,<13.0.0',
 'wget>=3.2,<4.0']

entry_points = \
{'console_scripts': ['onboardme = onboardme:main']}

setup_kwargs = {
    'name': 'onboardme',
    'version': '0.14.21',
    'description': 'An onboarding tool to install dot files and packages including a default mode with sensible defaults to run on most Debian/macOS machines.',
    'long_description': '## ☁️  onboard**me** 💻\n\n[![./docs/onboardme/screenshots/help_text.svg alt=\'screenshot of full output of onboardme --help](https://raw.githubusercontent.com/jessebot/onboardme/main/docs/onboardme/screenshots/help_text.svg)](https://raw.githubusercontent.com/jessebot/onboardme/main/docs/onboardme/screenshots/help_text.svg)\n\nThis is a project to store config files, as well as programatically install core packages across several package managers that we need for development. A lot of this was amassed from many years of quickly looking into a thing™️ , jotting it down, and then just hoping we\'d remember why it was there later, so this is now a renewed effort in remembering all the thing™️s, and automating as much as possible. The idea is that it\'s faster, smaller, and easier to configure than it\'s ansible equivalent.\n\nHere\'s an example of the terminal after the script has run:\n\n[![./docs/onboardme/screenshots/terminal_screenshot.png alt=\'screenshot of terminal after runnign onboardme. includes colortest-256, powerline prompt, icons for files in ls output, and syntax highlighting examples with cat command.\'](https://raw.githubusercontent.com/jessebot/onboardme/main/docs/onboardme/screenshots/terminal_screenshot.png)](https://raw.githubusercontent.com/jessebot/onboardme/main/docs/onboardme/screenshots/terminal_screenshot.png)\n\n\n## Quick Start\n[Get Started here](https://jessebot.github.io/onboardme/onboardme/getting-started) 💙!\n\n\n### Docs\nWe\'ve recently setup a justthedocs page [here](https://jessebot.github.io/onboardme/).\n\n\n## Features\nMade for the following OS (lastest stable):\n\n[![made-for-macOS](https://img.shields.io/badge/mac%20os-000000?style=for-the-badge&logo=apple&logoColor=white)](https://wikiless.org/wiki/MacOS?lang=en)\n[![made-for-debian](https://img.shields.io/badge/Debian-A81D33?style=for-the-badge&logo=debian&logoColor=white)](https://www.debian.org/)\n[![made-for-ubuntu](https://img.shields.io/badge/Ubuntu-E95420?style=for-the-badge&logo=ubuntu&logoColor=white)](https://ubuntu.com/)\n\nOptimized for:\n\n[![made-with-for-vim](https://img.shields.io/badge/VIM-%2311AB00.svg?&style=for-the-badge&logo=vim&logoColor=white)](https://www.vim.org/)\n[![made-with-python](https://img.shields.io/badge/Python-FFD43B?style=for-the-badge&logo=python&logoColor=blue)](https://www.python.org/)\n[![made-with-bash](https://img.shields.io/badge/GNU%20Bash-4EAA25?style=for-the-badge&logo=GNU%20Bash&logoColor=white)](https://www.gnu.org/software/bash/)\n\nAnd built using these great projects. Please go check them out:\n\n- [![rich](https://github.com/textualize/rich/raw/master/imgs/logo.svg)](https://github.com/Textualize/rich/tree/master)\n\n- [![nerd-fonts: Iconic font aggregator, collection, and patcher](https://raw.githubusercontent.com/ryanoasis/nerd-fonts/raw/master/images/nerd-fonts-logo.svg)](https://github.com/ryanoasis/nerd-fonts)\n\n- [powerline](https://powerline.readthedocs.io/en/master/overview.html)\n\n## Status\n\n🎉 Active project! Currently in later alpha :D (*But not yet stable. There be 🐛.*)\n\nPlease report 🐛 in the GitHub issues, and we will collect them, \nand release them into the wild, because we are vegan and nice.\n(Kidding, we will help! 😌)\n\n💙 We love contributors! Open a pull request, and we will review it asap!\n\n## Collaborators\n\n<!-- readme: collaborators -start -->\n<table>\n<tr>\n    <td align="center">\n        <a href="https://github.com/jessebot">\n            <img src="https://avatars.githubusercontent.com/u/2389292?v=4" width="100;" alt="jessebot"/>\n            <br />\n            <sub><b>JesseBot</b></sub>\n        </a>\n    </td>\n    <td align="center">\n        <a href="https://github.com/cloudymax">\n            <img src="https://avatars.githubusercontent.com/u/84841307?v=4" width="100;" alt="cloudymax"/>\n            <br />\n            <sub><b>Max!</b></sub>\n        </a>\n    </td></tr>\n</table>\n<!-- readme: collaborators -end -->\n\n## Shameless plugs for other projects\nLooking for a project to get running on a machine that has no OS at all?\nCheck out [pxeless](https://github.com/cloudymax/pxeless).\n\nLooking for a project to get started with self hosting k8s stuff?\nCheck out [smol k8s lab](https://github.com/jessebot/smol_k8s_lab).\n',
    'author': 'Jesse Hitch',
    'author_email': 'jessebot@linux.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'http://github.com/jessebot/onboardme',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
